import Vue from 'vue';
import warning from 'warning';
import { del, add } from './Suspense';
import { currentInstance, currentSuspenseInstance, setCurrentInstance } from './currentInstance';
import findSuspenseInstance from './findSuspenseInstance';
Vue.mixin({
    created: function () {
        setCurrentInstance(this);
    },
});
//Vue.observable works in 2.6+
function observable(data) {
    if (Vue.observable) {
        return Vue.observable(data);
    }
    return new Vue({
        data: function () {
            return data;
        },
    }).$data;
}
export default function CreateResource(fetchFactory, options) {
    var $res = observable({
        $$result: null,
        $$error: null,
        $$loading: false,
        $$loaded: false, // it has loaded data successed once more
    });
    var currInstance = currentInstance;
    fetchFactory.suspenseInstance = currentSuspenseInstance
        ? currentSuspenseInstance
        : findSuspenseInstance(currInstance);
    var hasSuspenseInstance = !!fetchFactory.suspenseInstance;
    var resourceResult = {
        read: function (input) {
            // prevent
            if (options && options.prevent && $res.$$loading) {
                return $res.$$promiser;
            }
            $res.$$loading = true;
            // Because we don't need caching, this is just a unique identifier,
            // and each call to .read() is a completely new request.
            var uniqueWrapFactory = function (i) { return fetchFactory(i); };
            if (hasSuspenseInstance) {
                // Establish a relationship between the fetchFactory and the current component instance
                uniqueWrapFactory.suspenseInstance = fetchFactory.suspenseInstance;
                add(uniqueWrapFactory);
            }
            // Start fetching asynchronous data
            var promise = ($res.$$promiser = uniqueWrapFactory(input));
            promise
                .then(function (res) {
                // Trigger update
                $res.$$result = options && options.onSuccess ? options.onSuccess(res) : res;
                if (!$res.$$loaded) {
                    $res.$$loaded = true;
                }
                if (hasSuspenseInstance) {
                    del(uniqueWrapFactory);
                }
            })
                .catch(function (err) {
                warning(process.env.NODE_ENV === 'production', err.message);
                $res.$$error = options && options.onError ? options.onError(err) : err;
                if (hasSuspenseInstance) {
                    del(uniqueWrapFactory, err);
                }
            })
                .finally(function () {
                $res.$$loading = false;
            });
            return promise;
        },
        get $result() {
            return $res.$$result;
        },
        set $result(val) {
            $res.$$result = val;
        },
        get $error() {
            return $res.$$error;
        },
        set $error(val) {
            $res.$$error = val;
        },
        get $loading() {
            return $res.$$loading;
        },
        get $loaded() {
            return $res.$$loaded;
        },
        fork: function () {
            return CreateResource(function (i) { return fetchFactory(i); }, options);
        },
    };
    return resourceResult;
}
//# sourceMappingURL=createResource.js.map